<?php

namespace FiloBlu\Refilo\Plugins\TargetRule\Model;

use FiloBlu\Refilo\Helper\Data;
use Magento\Framework\Indexer\IndexerRegistry;
use Magento\Framework\Indexer\StateInterface;
use Magento\TargetRule\Model\Rule;
use InvalidArgumentException;

class RulePlugin {

    /**
     * @var Data
     */
    protected $helperData;
    /**
     * @var IndexerRegistry
     */
    protected $indexerRegistry;

    /**
     * @param Data $helperData
     * @param IndexerRegistry $indexerRegistry
     */
    public function __construct(
        Data $helperData,
        IndexerRegistry $indexerRegistry
    )
    {
        $this->helperData = $helperData;
        $this->indexerRegistry = $indexerRegistry;
    }

    /**
     * @param Rule $subject
     * @param $result
     * @return mixed
     */
    public function afterGetPositionsLimit(Rule $subject, $result)
    {
        if($this->helperData->isTargetRuleOverrideLimitEnabled()){
            return $this->helperData->getTargetRuleOverrideLimit();
        }
        return $result;
    }

    /**
     * @param Rule $subject
     * @param $result
     * @return mixed
     * @throws \Exception
     */
    public function afterAfterSave(Rule $subject, $result)
    {
        if($this->helperData->isTargetRuleTriggerBeehiveEnabled()){
            $data = [
                'is_active',
                'from_date',
                'to_date',
                'conditions',
                'apply_to',
                'actions',
                'customer_segment_ids',
            ];
            if ($subject->isObjectNew() || $subject->dataHasChangedForAny($data)) {
                $this->resetReindex(['beehive_related']);
            }
        }
        return $result;
    }

    /**
     * @param Rule $subject
     * @param $result
     * @return mixed
     * @throws \Exception
     */
    public function afterDeleteCommit(Rule $subject, $result)
    {
        if($this->helperData->isTargetRuleTriggerBeehiveEnabled()){
            $this->resetReindex(['beehive_related']);
        }
        return $result;
    }


    /**
     * @param $indexers
     * @throws \Exception
     */
    protected function resetReindex($indexers)
    {
        foreach ($indexers as $name) {
            try {
                $indexer = $this->indexerRegistry->get($name);
                $state = $indexer->getState();
                $state->setStatus(StateInterface::STATUS_INVALID);
                $state->save();
                $indexer->setState($state);
            } catch (InvalidArgumentException $e) {
                continue;
            }
        }
    }

}
